==> geiger_localstorage-0.6.13.tar.gz: lib/src/db/data/node_value_implementation.dart <==
library geiger_localstorage;

import 'dart:collection';

import 'package:geiger_localstorage/src/db/data/timestamp.dart';
import 'package:geiger_localstorage/src/serialization/byte_sink.dart';
import 'package:geiger_localstorage/src/serialization/byte_stream.dart';
import 'package:geiger_localstorage/src/serialization/serializer_helper.dart';
import 'package:intl/locale.dart';

import 'language_range.dart';
import 'node_value.dart';

/// This abstract class defines the common attributes for all NodeValueObjects.
class NodeValueImpl implements NodeValue {
  static const int serialversionUID = 871283188;
  static final Locale defaultLocale = Locale.fromSubtags(languageCode: 'en');

  /// They key is used to identify the NodeValueObject inside a StorageNode,
  /// therefore, the key is unique within one StorageNode.
  String _key;
  final Map<String, String> _value = {};

  /// <p>The type of the value.</p>>
  String? _type;

  /// Description of this value, can be used for translation.
  final Map<String, String> _description = {};

  /// Defines the epoch when this value was last modified.
  late int _lastModified;

  /// <p>A fully fledged constructor for creating key/value pairs suitable for sharing.</p>
  ///
  /// @param key          the name of the key/value pair
  /// @param value        the value to be set
  /// @param type         a searchable type field
  /// @param description  the description to be shown when asking for consent of sharing
  /// @param lastModified the last modified date to be set
  NodeValueImpl(this._key, String value,
      [this._type, String? description, int? lastModified]) {
    setLocalizedString(_value, value, defaultLocale);
    if (description != null) {
      setLocalizedString(_description, description, defaultLocale);
    }
    _lastModified = lastModified ?? 0;
  }

  @override
  String get key {
    return _key;
  }

  set key(String key) {
    _key = key;
    touch();
  }

  @override
  String? getValue(String languageRange) {
    return getLocalizedString(_value, languageRange);
  }

  @override
  String get value {
    return getValue(defaultLocale.toLanguageTag())!;
  }

  @override
  Map<Locale, String> get allValueTranslations {
    Map<Locale, String> m = HashMap();
    for (var e in _value.entries) {
      m[Locale.parse(e.key)] = e.value;
    }
    return m;
  }

  @override
  void setValue(String value, [Locale? locale]) {
    setLocalizedString(_value, value, locale ?? defaultLocale);
    touch();
  }

  @override
  String? get type {
    return _type;
  }

  @override
  set type(String? type) {
    _type = type;
    touch();
  }

  @override
  String? getDescription([String? languageRange]) {
    return getLocalizedString(
        _description, languageRange ?? defaultLocale.toLanguageTag());
  }

  @override
  Map<Locale, String> get allDescriptionTranslations {
    Map<Locale, String> m = HashMap();
    for (var e in _description.entries) {
      m[Locale.parse(e.key)] = e.value;
    }
    return m;
  }

  static Locale lookupLocale(Map<String, String> map, String languageRange) {
    if (map.isEmpty) {
      return defaultLocale;
    }

    // Get Language Range
    var lr = LanguageRange.parse(languageRange);

    // extending range list with locale only
    return Locale.parse(lr.getBestLanguage(map));
  }

  static String? getLocalizedString(
      Map<String, String> map, String languageRange) {
    return map[lookupLocale(map, languageRange).toLanguageTag()];
  }

  static void setLocalizedString(
      Map<String, String> map, String value, Locale locale) {
    if ((getLocalizedString(map, defaultLocale.toLanguageTag()) == null) &&
        (!(locale.toLanguageTag() == defaultLocale.toLanguageTag()))) {
      throw Exception('undefined string for locale ' +
          defaultLocale.toString() +
          ' Locale ' +
          locale.toLanguageTag());
    }
    map[locale.toLanguageTag()] = value;
  }

  @override
  String? setDescription(String description, [Locale? locale]) {
    locale ??= defaultLocale;
    var ret = getLocalizedString(_description, locale.toLanguageTag());
    setLocalizedString(_description, description, locale);
    touch();
    return ret;
  }

  @override
  int get lastModified {
    return _lastModified;
  }

  @override
  set lastModified(int timestamp) {
    _lastModified = timestamp;
  }

  @override
  void update(NodeValue nodeValue) {
    NodeValueImpl n2 = nodeValue as NodeValueImpl;
    _key = n2.key;
    _value.clear();
    for (var e in n2._value.entries) {
      _value[e.key] = e.value;
    }
    _type = n2.type;
    _description.clear();
    for (var e in n2._description.entries) {
      _description[e.key] = e.value;
    }
    touch();
  }

  @override
  void touch() {
    lastModified = ExtendedTimestamp.now(false);
  }

  @override
  NodeValue deepClone() {
    NodeValue ret = NodeValueImpl(key, value);
    ret.update(this);
    return ret;
  }

  /// <p>prints a space prefixed representation of the NodeValue.</p>
  ///
  /// @param prefix a prefix (typically a series of spaces
  /// @return the string representation
  @override
  String toString([String prefix = '']) {
    var sb = StringBuffer();
    sb.write(prefix);
    sb.write(key);
    if (type != null) {
      sb.write(':');
      sb.write(type);
    }
    sb.write('={');
    if (_value.length == 1) {
      sb.write(defaultLocale.toLanguageTag());
      sb.write('=>"');
      sb.write(_value[defaultLocale.toLanguageTag()]);
      sb.write('"}');
    } else {
      sb.write('\n');
      var i = 0;
      List<String> keyList = List.from(_value.keys);
      keyList.sort();
      for (String l in keyList) {
        if (i > 0) {
          sb.write(',');
          sb.write('\n');
        }
        sb.write(prefix);
        sb.write('  ');
        sb.write(Locale.parse(l).toLanguageTag());
        sb.write('=>"');
        sb.write(_value[l]);
        sb.write('"');
        i++;
      }
      sb.write('\n');
      sb.write(prefix);
      sb.write('}');
    }
    return sb.toString();
  }

  @override
  bool operator ==(Object other) => equals(other);

  @override
  bool equals(Object? object) {
    if (object == null || object is! NodeValueImpl) {
      return false;
    }
    return toString() == object.toString();
  }

  @override
  int get hashCode => toString().hashCode;

  @override
  void toByteArrayStream(ByteSink out) async {
    SerializerHelper.writeLong(out, serialversionUID);
    SerializerHelper.writeString(out, key);
    SerializerHelper.writeInt(out, value.length);
    for (MapEntry<String, String> e in _value.entries) {
      SerializerHelper.writeString(out, e.key);
      SerializerHelper.writeString(out, e.value);
    }
    SerializerHelper.writeString(out, type);
    SerializerHelper.writeLong(out, lastModified);
    SerializerHelper.writeInt(out, _description.length);
    for (MapEntry<String, String> e in _description.entries) {
      SerializerHelper.writeString(out, e.key);
      SerializerHelper.writeString(out, e.value);
    }
    SerializerHelper.writeLong(out, serialversionUID);
  }

  /// <p>Deserializes a NodeValue from a byteStream.</p>
  /// @param in the stream to be read
  /// @return the deserialized NodeValue
  /// @throws IOException if an exception happens deserializing the stream
  static Future<NodeValueImpl> fromByteArrayStream(ByteStream in_) async {
    if (await SerializerHelper.readLong(in_) != serialversionUID) {
      throw Exception('failed to parse NodeValueImpl (bad stream?)');
    }
    var nv = NodeValueImpl(await SerializerHelper.readString(in_) ?? '', '');
    int counter = await SerializerHelper.readInt(in_);
    nv._value.clear();
    for (var i = 0; i < counter; i++) {
      nv._value[Locale.parse(await SerializerHelper.readString(in_) ?? '')
          .toLanguageTag()] = await SerializerHelper.readString(in_) ?? '';
    }
    nv._type = await SerializerHelper.readString(in_);
    nv._lastModified = await SerializerHelper.readLong(in_);
    counter = await SerializerHelper.readInt(in_);
    for (var i = 0; i < counter; i++) {
      nv._description[Locale.parse(await SerializerHelper.readString(in_) ?? '')
          .toLanguageTag()] = await SerializerHelper.readString(in_) ?? '';
    }
    if (await SerializerHelper.readLong(in_) != serialversionUID) {
      throw Exception('failed to parse NodeValueImpl (bad stream end?)');
    }
    return nv;
  }

  @override
  String get extendedLastModified {
    return ExtendedTimestamp.convertToExtendedTimestamp(lastModified);
  }

  @override
  String? toSimpleString([String? languageRange]) {
    languageRange ??= NodeValueImpl.defaultLocale.languageCode;
    return getValue(languageRange);
  }
}

==> geiger_localstorage-0.6.17.tar.gz: lib/src/db/data/node_value_implementation.dart <==
library geiger_localstorage;

import 'dart:collection';

import 'package:geiger_localstorage/src/db/data/timestamp.dart';
import 'package:geiger_localstorage/src/serialization/byte_sink.dart';
import 'package:geiger_localstorage/src/serialization/byte_stream.dart';
import 'package:geiger_localstorage/src/serialization/serializer_helper.dart';
import 'package:intl/locale.dart';

import 'language_range.dart';
import 'node_value.dart';

/// This abstract class defines the common attributes for all NodeValueObjects.
class NodeValueImpl implements NodeValue {
  static const int serialversionUID = 871283188;
  static final Locale defaultLocale = Locale.fromSubtags(languageCode: 'en');

  /// They key is used to identify the NodeValueObject inside a StorageNode,
  /// therefore, the key is unique within one StorageNode.
  String _key;
  final Map<String, String> _value = {};

  /// <p>The type of the value.</p>>
  String? _type;

  /// Description of this value, can be used for translation.
  final Map<String, String> _description = {};

  /// Defines the epoch when this value was last modified.
  late int _lastModified;

  /// <p>A fully fledged constructor for creating key/value pairs suitable for sharing.</p>
  ///
  /// @param key          the name of the key/value pair
  /// @param value        the value to be set
  /// @param type         a searchable type field
  /// @param description  the description to be shown when asking for consent of sharing
  /// @param lastModified the last modified date to be set
  NodeValueImpl(this._key, String value,
      [this._type, String? description, int? lastModified]) {
    setLocalizedString(_value, value, defaultLocale);
    if (description != null) {
      setLocalizedString(_description, description, defaultLocale);
    }
    _lastModified = lastModified ?? 0;
  }

  @override
  String get key {
    return _key;
  }

  set key(String key) {
    _key = key;
    touch();
  }

  @override
  String? getValue(String languageRange) {
    return getLocalizedString(_value, languageRange);
  }

  @override
  String get value {
    return getValue(defaultLocale.toLanguageTag())!;
  }

  @override
  Map<Locale, String> get allValueTranslations {
    Map<Locale, String> m = HashMap();
    for (var e in _value.entries) {
      m[Locale.parse(e.key)] = e.value;
    }
    return m;
  }

  @override
  void setValue(String value, [Locale? locale]) {
    setLocalizedString(_value, value, locale ?? defaultLocale);
    touch();
  }

  @override
  String? get type {
    return _type;
  }

  @override
  set type(String? type) {
    _type = type;
    touch();
  }

  @override
  String? getDescription([String? languageRange]) {
    return getLocalizedString(
        _description, languageRange ?? defaultLocale.toLanguageTag());
  }

  @override
  Map<Locale, String> get allDescriptionTranslations {
    Map<Locale, String> m = HashMap();
    for (var e in _description.entries) {
      m[Locale.parse(e.key)] = e.value;
    }
    return m;
  }

  static Locale lookupLocale(Map<String, String> map, String languageRange) {
    if (map.isEmpty) {
      return defaultLocale;
    }

    // Get Language Range
    var lr = LanguageRange.parse(languageRange);

    // extending range list with locale only
    return Locale.parse(lr.getBestLanguage(map));
  }

  static String? getLocalizedString(
      Map<String, String> map, String languageRange) {
    return map[lookupLocale(map, languageRange).toLanguageTag()];
  }

  static void setLocalizedString(
      Map<String, String> map, String value, Locale locale) {
    if ((getLocalizedString(map, defaultLocale.toLanguageTag()) == null) &&
        (!(locale.toLanguageTag() == defaultLocale.toLanguageTag()))) {
      throw Exception('undefined string for locale ' +
          defaultLocale.toString() +
          ' Locale ' +
          locale.toLanguageTag());
    }
    map[locale.toLanguageTag()] = value;
  }

  @override
  String? setDescription(String description, [Locale? locale]) {
    locale ??= defaultLocale;
    var ret = getLocalizedString(_description, locale.toLanguageTag());
    setLocalizedString(_description, description, locale);
    touch();
    return ret;
  }

  @override
  int get lastModified {
    return _lastModified;
  }

  @override
  set lastModified(int timestamp) {
    _lastModified = timestamp;
  }

  @override
  void update(NodeValue nodeValue) {
    NodeValueImpl n2 = nodeValue as NodeValueImpl;
    _key = n2.key;
    _value.clear();
    for (var e in n2._value.entries) {
      _value[e.key] = e.value;
    }
    _type = n2.type;
    _description.clear();
    for (var e in n2._description.entries) {
      _description[e.key] = e.value;
    }
    touch();
  }

  @override
  void touch() {
    lastModified = ExtendedTimestamp.now(false);
  }

  @override
  NodeValue deepClone() {
    NodeValue ret = NodeValueImpl(key, value);
    ret.update(this);
    return ret;
  }

  /// <p>prints a space prefixed representation of the NodeValue.</p>
  ///
  /// @param prefix a prefix (typically a series of spaces
  /// @return the string representation
  @override
  String toString([String prefix = '']) {
    var sb = StringBuffer();
    sb.write(prefix);
    sb.write(key);
    if (type != null) {
      sb.write(':');
      sb.write(type);
    }
    sb.write('={');
    List<String> keyList = List.from(_value.keys);
    keyList.sort();
    // write all values
    if (keyList.length > 1) {
      sb.write('\n');
    }
    for (int i = 0; i < keyList.length; i++) {
      if (i > 0) {
        sb.write(',');
        sb.write('\n');
      }
      if (keyList.length > 1) {
        sb.write(prefix);
        sb.write('  ');
      }
      sb.write(Locale.parse(keyList[i]).toLanguageTag());
      sb.write('=>"');
      sb.write(_value[keyList[i]]);
      sb.write('"');
      if (keyList.length > 1) {
        sb.write('\n');
        sb.write(prefix);
      }
    }
    sb.write('}');
    return sb.toString();
  }

  @override
  bool operator ==(Object other) => equals(other);

  @override
  bool equals(Object? object) {
    if (object == null || object is! NodeValueImpl) {
      return false;
    }
    return toString() == object.toString();
  }

  @override
  int get hashCode => toString().hashCode;

  @override
  void toByteArrayStream(ByteSink out) async {
    SerializerHelper.writeLong(out, serialversionUID);
    SerializerHelper.writeString(out, key);
    SerializerHelper.writeInt(out, value.length);
    for (MapEntry<String, String> e in _value.entries) {
      SerializerHelper.writeString(out, e.key);
      SerializerHelper.writeString(out, e.value);
    }
    SerializerHelper.writeString(out, type);
    SerializerHelper.writeLong(out, lastModified);
    SerializerHelper.writeInt(out, _description.length);
    for (MapEntry<String, String> e in _description.entries) {
      SerializerHelper.writeString(out, e.key);
      SerializerHelper.writeString(out, e.value);
    }
    SerializerHelper.writeLong(out, serialversionUID);
  }

  /// <p>Deserializes a NodeValue from a byteStream.</p>
  /// @param in the stream to be read
  /// @return the deserialized NodeValue
  /// @throws IOException if an exception happens deserializing the stream
  static Future<NodeValueImpl> fromByteArrayStream(ByteStream in_) async {
    if (await SerializerHelper.readLong(in_) != serialversionUID) {
      throw Exception('failed to parse NodeValueImpl (bad stream?)');
    }
    var nv = NodeValueImpl(await SerializerHelper.readString(in_) ?? '', '');
    int counter = await SerializerHelper.readInt(in_);
    nv._value.clear();
    for (var i = 0; i < counter; i++) {
      nv._value[Locale.parse(await SerializerHelper.readString(in_) ?? '')
          .toLanguageTag()] = await SerializerHelper.readString(in_) ?? '';
    }
    nv._type = await SerializerHelper.readString(in_);
    nv._lastModified = await SerializerHelper.readLong(in_);
    counter = await SerializerHelper.readInt(in_);
    for (var i = 0; i < counter; i++) {
      nv._description[Locale.parse(await SerializerHelper.readString(in_) ?? '')
          .toLanguageTag()] = await SerializerHelper.readString(in_) ?? '';
    }
    if (await SerializerHelper.readLong(in_) != serialversionUID) {
      throw Exception('failed to parse NodeValueImpl (bad stream end?)');
    }
    return nv;
  }

  @override
  String get extendedLastModified {
    return ExtendedTimestamp.convertToExtendedTimestamp(lastModified);
  }

  @override
  String? toSimpleString([String? languageRange]) {
    languageRange ??= NodeValueImpl.defaultLocale.languageCode;
    return getValue(languageRange);
  }
}
